@@ -1,11 +1,15 @@
 package web
 
 import (
+	"context"
+	"fmt"
+	"io"
+
 	"github.com/a-h/templ"
 
 	"github.com/TecharoHQ/anubis/lib/challenge"
+	"github.com/TecharoHQ/anubis/lib/config"
 	"github.com/TecharoHQ/anubis/lib/localization"
-	"github.com/TecharoHQ/anubis/lib/policy/config"
 )
 
 func Base(title string, body templ.Component, impressum *config.Impressum, localizer *localization.SimpleLocalizer) templ.Component {
@@ -29,3 +33,10 @@ func ErrorPage(msg, mail, code string, localizer *localization.SimpleLocalizer)
 func Bench(localizer *localization.SimpleLocalizer) templ.Component {
 	return bench(localizer)
 }
+
+func honeypotLink(href string) templ.Component {
+	return templ.ComponentFunc(func(ctx context.Context, w io.Writer) error {
+		fmt.Fprintf(w, `<script type="ignore"><a href="%s">Don't click me</a></script>`, href)
+		return nil
+	})
+}
